/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka;

import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.serialization.JacksonDeserializer;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.common.errors.WakeupException;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.log4j.Logger;

import javax.annotation.PreDestroy;
import java.util.Collections;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

public class CbsKafkaConsumer {
    private static final Logger log = Logger.getLogger(CbsKafkaConsumer.class);
    private final AtomicBoolean closed = new AtomicBoolean(false);
    private final Consumer<String, DataFetcherJob> consumer;

    public CbsKafkaConsumer(String topic, Properties kafkaProperties, CbsKafkaFactory<String, DataFetcherJob> cbsKafkaFactory) {
        if (kafkaProperties.contains(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG))
            log.warn("Kafka property " + ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG + " not supported. Ignoring");
        if (kafkaProperties.contains(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG))
            log.warn("Kafka property " + ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG + " not supported. Ignoring");

        consumer = cbsKafkaFactory.getNSPKafkaConsumer(kafkaProperties, new StringDeserializer(), new JacksonDeserializer<>(DataFetcherJob.class));
        consumer.subscribe(Collections.singletonList(topic));
    }

    /**
     * Shutdown handling per KafkaConsumer JavaDoc section "Multi-threaded Processing"
     *
     * @see org.apache.kafka.clients.consumer.KafkaConsumer
     */
    @PreDestroy
    public void shutdown() {
        if (!closed.get()) {
            log.info("Closing CBS Kafka Consumer");
            closed.set(true);
            consumer.wakeup(); //will generate a WakeupException on the thread (either aborts current poll or causes next call to poll to cause WakeupException)
        }
    }

    public synchronized Set<DataFetcherJob> getJobs() { //todo remove 'synchronized' and replace with a pool of consumers
        log.debug("getJobs called");
        ConsumerRecords<String, DataFetcherJob> consumerRecords;
        try {
            consumerRecords = consumer.poll(100);
        } catch (WakeupException e) {
            log.debug("WakeupException was raised");
            if (!closed.get()) throw e;
            consumer.close();
            return Collections.emptySet();
        }
        log.info(consumerRecords.count() + " record(s) recieved from Kafka");

        if (log.isDebugEnabled()) {
            for (ConsumerRecord<String, DataFetcherJob> record : consumerRecords) {
                log.debug("---------\n" +
                        "Record " + record.key() + " received:\n" +
                        "\t Value:" + record.value() + "\n" +
                        "\t Topic:" + record.topic() + "\n" +
                        "\t Partition:" + record.partition() + "\n" +
                        "\t Offset:" + record.offset());
            }
        }

        Set<DataFetcherJob> jobs = new HashSet<>(consumerRecords.count());
        consumerRecords.forEach(record -> jobs.add(record.value()));
        return jobs;
    }

}
