/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka;

import dk.sds.nsp.kafka.clients.consumer.NSPKafkaConsumer;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.JobType;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.Deserializer;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;

import java.util.*;

import static com.google.common.truth.Truth.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.when;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class CbsKafkaConsumerTest {
    @Mock
    private NSPKafkaConsumer<String, DataFetcherJob> mockNspConsumer;
    @Mock
    private CbsKafkaFactory<String, DataFetcherJob> mockFactory;
    @Captor
    private ArgumentCaptor<ProducerRecord<String, DataFetcherJob>> producerRecordArgumentCaptor;
    private CbsKafkaConsumer cbsKafkaConsumer;
    private static final String topic = "topic";

    @BeforeEach
    void setUp() {
        when(mockFactory.getNSPKafkaConsumer(any(Properties.class), any(Deserializer.class), any(Deserializer.class))).thenReturn(mockNspConsumer);
        cbsKafkaConsumer = new CbsKafkaConsumer(topic, new Properties(), mockFactory);
    }

    @Test
    void test_get_single_job() {
        Map<TopicPartition, List<ConsumerRecord<String, DataFetcherJob>>> records = new HashMap<>();
        ArrayList<ConsumerRecord<String, DataFetcherJob>> record = new ArrayList<>();
        DataFetcherJob job = createRandomDataFetcherJob();
        record.add(new ConsumerRecord<>(topic, 0, 0, job.getCpr(), job));
        records.put(new TopicPartition(topic, 0), record);
        when(mockNspConsumer.poll(anyLong())).thenReturn(new ConsumerRecords<>(records));

        Set<DataFetcherJob> jobs = cbsKafkaConsumer.getJobs();
        assertThat(jobs).containsExactly(job);
    }

    @Test
    void test_get_multiple_jobs() {
        Map<TopicPartition, List<ConsumerRecord<String, DataFetcherJob>>> records = new HashMap<>();
        ArrayList<ConsumerRecord<String, DataFetcherJob>> record = new ArrayList<>();
        DataFetcherJob job1 = createRandomDataFetcherJob();
        DataFetcherJob job2 = createRandomDataFetcherJob();
        record.add(new ConsumerRecord<>(topic, 0, 0, job1.getCpr(), job1));
        record.add(new ConsumerRecord<>(topic, 0, 0, job2.getCpr(), job2));
        records.put(new TopicPartition(topic, 0), record);
        when(mockNspConsumer.poll(anyLong())).thenReturn(new ConsumerRecords<>(records));

        Set<DataFetcherJob> jobs = cbsKafkaConsumer.getJobs();
        assertThat(jobs).containsExactly(job1, job2);

    }

    @Test
    void identical_records_are_collapsed() {
        Map<TopicPartition, List<ConsumerRecord<String, DataFetcherJob>>> recordMap = new HashMap<>();
        ArrayList<ConsumerRecord<String, DataFetcherJob>> records = new ArrayList<>();
        DataFetcherJob job1 = createRandomDataFetcherJob();
        String cpr = job1.getCpr();
        DataFetcherJob job2 = DataFetcherJobBuilder.aDataFetcherJob()
                .withCpr(cpr)
                .withType(job1.getType())
                .withMsgId(job1.getMsgId())
                .withJobId(job1.getJobId())
                .build();

        records.add(new ConsumerRecord<>(topic, 0, 0, cpr, job1));
        records.add(new ConsumerRecord<>(topic, 0, 0, cpr, job2));
        recordMap.put(new TopicPartition(topic, 0), records);
        when(mockNspConsumer.poll(anyLong())).thenReturn(new ConsumerRecords<>(recordMap));

        Set<DataFetcherJob> jobs = cbsKafkaConsumer.getJobs();
        assertThat(jobs).hasSize(1);
        assertThat(jobs).containsExactly(job1);
        assertThat(jobs).containsExactly(job2);
    }

    public static DataFetcherJob createRandomDataFetcherJob() {
        int pickRandomEnum = new Random().nextInt(JobType.values().length);
        JobType randomJobType = JobType.values()[pickRandomEnum];

        return DataFetcherJobBuilder.aDataFetcherJob()
                .withCpr(TestUtils.randomCprNumber())
                .withType(randomJobType)
                .withMsgId(UUID.randomUUID().toString())
                .withJobId(UUID.randomUUID().toString())
                .build();
    }
}
