/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.UUID;

import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordImpl.CREATION_TIME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordImpl.DATA_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordImpl.INVALIDATION_TIME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordImpl.KEY_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordImpl.TRACE_ID_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.RecordMappers.mapper;
import static dk.sundhedsdatastyrelsen.fluent.spring.jdbc.UUIDs.toBytes;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class RecordMappersTest {

    @Test
    void test() throws SQLException {
        final UUID key = UUID.randomUUID();
        final ResultSet resultSet = mock(ResultSet.class);
        final LocalDateTime creationTime = LocalDateTime.now();
        final LocalDateTime invalidationTime = LocalDateTime.now();
        final String data = "DATA";
        final String traceId = "TRACE_ID";

        when(resultSet.getBytes(KEY_COLUMN_LABEL)).thenReturn(toBytes(key));
        when(resultSet.getTimestamp(CREATION_TIME_COLUMN_LABEL)).thenReturn(Timestamp.valueOf(creationTime));
        when(resultSet.getTimestamp(INVALIDATION_TIME_COLUMN_LABEL)).thenReturn(Timestamp.valueOf(invalidationTime));
        when(resultSet.getCharacterStream(DATA_COLUMN_LABEL)).thenReturn(new StringReader(data));
        when(resultSet.getString(TRACE_ID_COLUMN_LABEL)).thenReturn(traceId);

        final MyRecord record = mapper(MyRecord::new, RecordMappersTest::toString)
            .build()
            .mapRow(resultSet, 0);

        assertNotNull(record);
        assertEquals(key, record.key());
        assertEquals(creationTime, record.creationTime());
        assertEquals(invalidationTime, record.invalidationTime());
        assertEquals(data, record.data());
        assertEquals(traceId, record.traceId());
    }

    private static String toString(Reader reader) {
        try {
            return IOUtils.toString(reader);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static final class MyRecord extends RecordImpl<String, MyRecord> {}
}
