/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sds.nsp.security.Security;
import dk.sds.nsp.security.SecurityContext;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.JobType;
import dk.sundhedsdatastyrelsen.cbs.service.ErrorMessages;
import dk.sundhedsdatastyrelsen.cbs.service.Service;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.CoronaPassPort;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.GetCoronaPassRequest;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.GetCoronaPassResponse;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.cxf.annotations.SchemaValidation;
import org.apache.log4j.Logger;

import javax.annotation.Resource;
import javax.jws.WebMethod;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.ws.WebServiceContext;
import java.util.Optional;
import java.util.Random;

import static dk.sundhedsdatastyrelsen.cbs.service.servlet.ServiceServletContextListener.SPRING_APPLICATION_CONTEXT;

@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@WebService(name = "CoronaPassPort", targetNamespace = "http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service")
@SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
@XmlSeeAlso({
        ObjectFactory.class
})
public class CoronaPassWebService implements CoronaPassPort {
    private static final Logger logger = Logger.getLogger(CoronaPassWebService.class);

    @Resource
    public WebServiceContext context;

    @Override
    @WebMethod(operationName = "GetCoronaPass")
    @WebResult(name = "GetCoronaPassResponse", targetNamespace = "http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service", partName = "GetCoronaPassResponse")
    public GetCoronaPassResponse getCoronaPass(GetCoronaPassRequest request) {
        SecurityContext securityContext = Security.getSecurityContext();
        checkSecurityTicket(securityContext);
        String cprFromTicket = securityContext.getActingUser().get().getIdentifier();
        if (!cprFromTicket.equalsIgnoreCase(request.getIdentifier())) {
            logger.info("IDWS ticket CPR does not match request CPR");
        }
        String idwsMessageId = securityContext.getMessage().orElseThrow(RuntimeException::new).getIdentifier().orElseThrow(RuntimeException::new);

        logger.info("IDWS ticket CPR = " + cprFromTicket + ", SecurityContext = " + ReflectionToStringBuilder.toString(securityContext));

        // MOCKING BEGINS HERE
        if (Boolean.parseBoolean(System.getProperty("test.cache.hit", String.valueOf(new Random().nextBoolean())))) {
            logger.info("Mocking cache hit");
            return SPRING_APPLICATION_CONTEXT.getBean(MockDataService.class).cacheHit();
        } else {
            logger.info("Mocking cache miss");
            int pickRandomEnum = new Random().nextInt(JobType.values().length);
            JobType jobType = JobType.values()[pickRandomEnum];

            Service service = SPRING_APPLICATION_CONTEXT.getBean(Service.class);
            String jobId = service.createDataFetcherJob(cprFromTicket, idwsMessageId, jobType);
            return SPRING_APPLICATION_CONTEXT.getBean(MockDataService.class).cacheMiss(jobId);
        }

    }

    protected void checkSecurityTicket(SecurityContext securityContext) {
        Optional<SecurityContext.Ticket> ticketOptional = securityContext.getTicket();
        if (!ticketOptional.isPresent()) {
            throw new RuntimeException(ErrorMessages.NO_SECURITY_TICKET);
        }
        SecurityContext.Ticket ticket = ticketOptional.get();
        if (!ticket.isValid()) {
            throw new RuntimeException(ErrorMessages.INVALID_SECURITY_TICKET);
        }
    }

}
