/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.fluent.spring.jdbc;

import org.springframework.jdbc.core.RowMapper;
import org.springframework.lang.NonNull;

import java.sql.ResultSet;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Supplier;

public final class Mappers {

    private Mappers() {}

    @NonNull
    public static <T> RowMapperBuilder<T> mapper(@NonNull final Supplier<T> constructor) {
        return new RowMapperBuilder<>(constructor);
    }

    public static final class RowMapperBuilder<T> {

        @NonNull private final Supplier<T> constructor;
        @NonNull private final List<BiConsumer<T, ResultSet>> deserializers;

        private RowMapperBuilder(@NonNull final Supplier<T> constructor) {
            this.constructor = constructor;
            deserializers = new ArrayList<>();
        }

        @NonNull
        public <U> RowMapperBuilder<T> addMapping(
            @NonNull final String columnLabel,
            @NonNull final Deserializer<T, U> deserializer) {
            deserializers.add(
                (obj, resultSet) -> deserializer.setter.accept(obj, deserializer.mapper.apply(resultSet, columnLabel)));
            return this;
        }

        @NonNull
        public RowMapperBuilder<T> addTimestampMapping(
            @NonNull final String columnLabel,
            @NonNull final BiConsumer<T, Timestamp> setter) {
            addMapping(columnLabel, timestamp(setter));
            return this;
        }

        @NonNull
        public RowMapperBuilder<T> addTimestampToLocalDateTimeMapping(
            @NonNull final String columnLabel,
            @NonNull final BiConsumer<T, LocalDateTime> setter) {
            addMapping(columnLabel, timestampToLocalDateTime(setter));
            return this;
        }

        @NonNull
        public RowMapperBuilder<T> addStringMapping(
            @NonNull final String columnLabel,
            @NonNull final BiConsumer<T, String> setter) {
            addMapping(columnLabel, string(setter));
            return this;
        }

        @NonNull
        public RowMapperBuilder<T> addIntegerMapping(
            @NonNull final String columnLabel,
            @NonNull final BiConsumer<T, Integer> setter) {
            addMapping(columnLabel, integer(setter));
            return this;
        }

        @NonNull
        public RowMapperBuilder<T> addBytesMapping(
            @NonNull final String columnLabel,
            @NonNull final BiConsumer<T, UUID> setter) {
            addMapping(columnLabel, bytes(setter));
            return this;
        }

        @NonNull
        public RowMapper<T> build() {
            return new Mapper<>(constructor, deserializers);
        }
    }

    private static final class Mapper<T> implements RowMapper<T> {

        @NonNull private final Supplier<T> constructor;
        @NonNull private final List<BiConsumer<T, ResultSet>> fieldMappers;

        private Mapper(
            @NonNull final Supplier<T> constructor,
            @NonNull final List<BiConsumer<T, ResultSet>> fieldMappers) {
            this.constructor = constructor;
            this.fieldMappers = fieldMappers;
        }

        @Override
        public T mapRow(final ResultSet rs, final int rowNum) {
            final T obj = constructor.get();
            fieldMappers.forEach(fieldMapper -> fieldMapper.accept(obj, rs));
            return obj;
        }
    }

    @NonNull
    public static <T> DeserializerChain<T> deserializer(@NonNull final BiFunction<ResultSet, String, T> function) {
        return new DeserializerChain<>(function);
    }

    public static final class DeserializerChain<T> {

        @NonNull private final BiFunction<ResultSet, String, T> mapper;

        private DeserializerChain(@NonNull final BiFunction<ResultSet, String, T> mapper) {
            this.mapper = mapper;
        }

        @NonNull
        public <U> DeserializerChain<U> map(@NonNull final Function<T, U> after) {
            return new DeserializerChain<>(mapper.andThen(after));
        }

        @NonNull
        public <U> Deserializer<U, T> set(@NonNull final BiConsumer<U, ? super T> setter) {
            return new Deserializer<>(mapper, setter);
        }
    }

    public static final class Deserializer<T, U> {

        @NonNull private final BiFunction<ResultSet, String, U> mapper;
        @NonNull private final BiConsumer<T, ? super U> setter;

        private Deserializer(
            @NonNull final BiFunction<ResultSet, String, U> mapper,
            @NonNull final BiConsumer<T, ? super U> setter) {
            this.mapper = mapper;
            this.setter = setter;
        }
    }

    @NonNull
    public static <T> Deserializer<T, Timestamp> timestamp(@NonNull final BiConsumer<T, Timestamp> setter) {
        return deserializer(ResultSets::getTimestamp).set(setter);
    }

    @NonNull
    public static <T> Deserializer<T, LocalDateTime> timestampToLocalDateTime(@NonNull final BiConsumer<T, LocalDateTime> setter) {
        return deserializer(ResultSets::getTimestamp).map(Timestamp::toLocalDateTime).set(setter);
    }

    @NonNull
    public static <T> Deserializer<T, String> string(@NonNull final BiConsumer<T, String> setter) {
        return deserializer(ResultSets::getString).set(setter);
    }

    @NonNull
    public static <T> Deserializer<T, Integer> integer(@NonNull final BiConsumer<T, Integer> setter) {
        return deserializer(ResultSets::getInteger).set(setter);
    }

    @NonNull
    public static <T> Deserializer<T, UUID> bytes(@NonNull final BiConsumer<T, UUID> setter) {
        return deserializer(ResultSets::getBytes).map(UUIDs::of).set(setter);
    }
}
