/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.fluent.spring.jdbc;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.io.Reader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

/**
 * Utility class wrapping ResultSet operations to throw runtime exception {@link SQLRuntimeException} instead of the
 * checked exception {@link SQLException}, thus easing use in lambda expressions.
 */
public final class ResultSets {

    private ResultSets() {}

    /**
     * Retrieves the value of the designated column in the current row of a ResultSet object as a
     * <code>java.sql.Timestamp</code> object in the Java programming language.
     * <pre>Wrapper for {@link ResultSet#getTimestamp(String)}</pre>
     * @param resultSet the ResultSet object
     * @param columnLabel the label for the column specified with the SQL AS clause. If the SQL AS clause was not
     *                    specified, then the label is the name of the column
     * @return the column value; if the value is SQL <code>NULL</code>, the value returned is <code>null</code>
     * @throws SQLRuntimeException if the columnLabel is not valid; if a database access error occurs or this method is
     *                             called on a closed result set
     */
    @Nullable
    public static Timestamp getTimestamp(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            return resultSet.getTimestamp(columnLabel);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    /**
     * Retrieves the value of the designated column in the current row of a ResultSet object as a byte array in the
     * Java programming language. The bytes represent the raw values returned by the driver.
     * <pre>Wrapper for {@link ResultSet#getTimestamp(String)}</pre>
     * @param resultSet the ResultSet object
     * @param columnLabel the label for the column specified with the SQL AS clause. If the SQL AS clause was not
     *                    specified, then the label is the name of the column
     * @return the column value; if the value is SQL <code>NULL</code>, the value returned is <code>null</code>
     * @throws SQLRuntimeException if the columnLabel is not valid; if a database access error occurs or this method
     *                             is called on a closed result set
     */
    @Nullable
    public static byte[] getBytes(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            return resultSet.getBytes(columnLabel);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    @Nullable
    public static String getString(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            return resultSet.getString(columnLabel);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    @Nullable
    public static Reader getCharacterStream(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            return resultSet.getCharacterStream(columnLabel);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    @Nullable
    public static Integer getInteger(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            final int value = resultSet.getInt(columnLabel);
            return resultSet.wasNull() ? null : value;
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }

    public static int getInt(@NonNull final ResultSet resultSet, @NonNull final String columnLabel) {
        try {
            return resultSet.getInt(columnLabel);
        } catch (SQLException e) {
            throw new SQLRuntimeException(e);
        }
    }
}
